package com.sprokets.apps;

import java.io.*;
import java.util.*;
import java.math.*;

import com.sprokets.schemas.invoice.*;

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

public class ProduceInvoice
{	
	private static final BigDecimal POSTAGE_PER_ITEM = new BigDecimal("1.20");
		
	private static void addProduct(InvoiceItems items, String code, int quantity)
		throws Exception
	{
		// Retrieve product
		String description = null;
		String location = null;
		BigDecimal unitPrice = null;
		if (code.equals("W1000"))
		{
			description = "Small Widget";
			unitPrice = new BigDecimal("10.50");
			location = "WAB10";
		} else if (code.equals("W2000"))
		{
			description = "Medium Widget";
			unitPrice = new BigDecimal("15.75");
			location = "WAB11";
		} else if (code.equals("W3000"))
		{
			description = "Large Widget";
			unitPrice = new BigDecimal("20.40");
			location = "WAB12";
		}
		else
		{
			throw new Exception("Product '" + code + "' does not exist.");
		}
		
		// Add invoice item
		InvoiceItem item = new InvoiceItem();
		item.setCode(code);
		item.setDescription(description);
		item.setWarehouseLocation(location);
		item.setUnitPrice(unitPrice);
		item.setQuantity(quantity);
		item.setExtPrice(unitPrice.multiply(new BigDecimal(quantity)));
		items.addItem(item);		
	}
	
	public static void main(String[] args)
	{
		try
		{		
			// Create the list of invoiced products
			InvoiceItems items = new InvoiceItems();
			addProduct(items, "W1000", 10);
			addProduct(items, "W2000", 2);
			addProduct(items, "W3000", 4);
			
			// Calculate product and postage totals
			BigDecimal productTotal = new BigDecimal(0);
			BigDecimal postageTotal = new BigDecimal(0);
			int itemCount = items.getItemCount();
			for(int i=0; i<itemCount; i++)
			{
				InvoiceItem item = items.getItem(i);
				productTotal = productTotal.add(item.getExtPrice());
				postageTotal = postageTotal.add(POSTAGE_PER_ITEM.multiply(new BigDecimal(item.getQuantity())));
			}
			BigDecimal total = productTotal.add(postageTotal);
			
			// Customer Address
			Address address = new Address();
			address.setName("Joe Smith");
			address.setStreet("123 XYZ Street");
			address.setCity("Harrogate");
			address.setState("North Yorkshire");
			address.setCountry("United Kindgom");
			address.setZip("HG1 XYZ");

			// Ordered by
			OrderedBy orderedBy = new OrderedBy();
			orderedBy.setAddress(address);
			orderedBy.setCustomerNo("CUS1200JS");
			orderedBy.setDate(new Date());
					
			// Create the Invoice
			Invoice invoice = new Invoice();
			invoice.setOrdernumber("1001");
			invoice.setItems(items);
			invoice.setNetProduct(productTotal);
			invoice.setPostage(postageTotal);
			invoice.setRep("Fred Blogs");
			invoice.setShipDate(new Date());
			invoice.setDeliveryAddress(address);
			invoice.setOrderedBy(orderedBy);
			invoice.setTotalShipment(total);
			invoice.setAmountCharged(total);
			
			// Write 'invoice.xml' XML document
			FileWriter writer = new FileWriter("invoice.xml");
			Marshaller marshaller = new Marshaller(writer);
			marshaller.setNSPrefixAtRoot(true);
			marshaller.setNamespaceMapping("whs", "http://www.sprokets.com/schemas/whouse");
			marshaller.marshal(invoice);
			writer.close();
		}
		catch (MarshalException e)
		{
			e.printStackTrace(System.out);
		}		
		catch (ValidationException e)
		{
			e.printStackTrace(System.out);
		}		
		catch (IOException e)
		{
			e.printStackTrace(System.out);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.out);
		}
	}
}