package com.sprokets.apps;

import java.io.*;
import java.util.*;
import java.math.*;

import com.sprokets.schemas.whouse.*;
import com.sprokets.schemas.invoice.*;
import com.sprokets.schemas.returns.*;
import com.sprokets.schemas.returns.types.*;

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;

public class ProduceReturns
{
	private static void addDamagedProduct(ReturnItems items, Product product, int quantity, String description)
	{
		ReturnItem item = new ReturnItem();
		item.setReason(TypeReason.DA);
		item.setDescription(description);
		item.setProduct(product);
		item.setQuantity(quantity);
		items.addItem(item);		
	}
	
	public static void main(String[] args)
	{		
		try
		{
			// Load 'invoice.xml' XML document into Invoice object
			FileReader reader = new FileReader("invoice.xml");
			Invoice invoice = (Invoice) Unmarshaller.unmarshal(Invoice.class, reader);
			reader.close();
		
			// Create returns list
			InvoiceItems invoiceItems = invoice.getItems();
			ReturnItems items = new ReturnItems();
			addDamagedProduct(items, invoiceItems.getItem(1), 2, "A damaged cog!");
			
			// Create Returns object
			Returns returns = new Returns();
			returns.setOrdernumber(invoice.getOrdernumber());
			returns.setCustomerNo(invoice.getOrderedBy().getCustomerNo());
			returns.setDate(new Date());
			returns.setItems(items);
			
			// Write 'returns.xml' XML document
			FileWriter writer = new FileWriter("returns.xml");
			Marshaller marshaller = new Marshaller(writer);
			marshaller.setNSPrefixAtRoot(true);
			marshaller.setMarshalExtendedType(false);
			marshaller.setNamespaceMapping("whs", "http://www.sprokets.com/schemas/whouse");
			marshaller.marshal(returns);
			writer.close();
		}
		catch (IOException e)
		{
			e.printStackTrace(System.out);
		}
		catch (MarshalException e)
		{
			e.printStackTrace(System.out);
		}
		catch (ValidationException e)
		{
			e.printStackTrace(System.out);
		}
	}		
}
					 